@@ -1,5 +1,6 @@
 import logging
 from collections import namedtuple
+
 from .main import special_command, RAW_QUERY
 
 TableInfo = namedtuple("TableInfo", ['checks', 'relkind', 'hasindex',
@@ -75,7 +76,7 @@ def list_roles(cur, pattern, verbose):
 
 
 @special_command('\\db', '\\db[+] [pattern]', 'List tablespaces.')
-def list_tablestpaces(cur, pattern, **_):
+def list_tablespaces(cur, pattern, **_):
     """
     Returns (title, rows, headers, status)
     """
@@ -192,13 +193,13 @@ def _find_extensions(cur, pattern):
 
 def _describe_extension(cur, oid):
     sql = '''
-        SELECT 	pg_catalog.pg_describe_object(classid, objid, 0)
+        SELECT  pg_catalog.pg_describe_object(classid, objid, 0)
                   AS "Object Description"
-        FROM 	pg_catalog.pg_depend
+        FROM    pg_catalog.pg_depend
         WHERE   refclassid = 'pg_catalog.pg_extension'::pg_catalog.regclass
                 AND refobjid = %s
                 AND deptype = 'e'
-        ORDER BY 1 '''
+        ORDER BY 1'''
     sql = cur.mogrify(sql, [oid])
     log.debug(sql)
     cur.execute(sql)
@@ -211,7 +212,8 @@ def list_objects(cur, pattern, verbose, relkinds):
     """
         Returns (title, rows, header, status)
 
-        This method is used by list_tables, list_views, and list_indexes
+        This method is used by list_tables, list_views, list_materialized views
+        and list_indexes
 
         relkinds is a list of strings to filter pg_class.relkind
 
@@ -276,6 +278,11 @@ def list_views(cur, pattern, verbose):
     return list_objects(cur, pattern, verbose, ['v', 's', ''])
 
 
+@special_command('\\dm', '\\dm[+] [pattern]', 'List materialized views.')
+def list_materialized_views(cur, pattern, verbose):
+    return list_objects(cur, pattern, verbose, ['m', 's', ''])
+
+
 @special_command('\\ds', '\\ds[+] [pattern]', 'List sequences.')
 def list_sequences(cur, pattern, verbose):
     return list_objects(cur, pattern, verbose, ['S', 's', ''])
@@ -448,38 +455,15 @@ def list_datatypes(cur, pattern, verbose):
 
 
 @special_command('describe', 'DESCRIBE [pattern]', '', hidden=True, case_sensitive=False)
-@special_command('\\d', '\\d [pattern]', 'List or describe tables, views and sequences.')
+@special_command('\\d', '\\d[+] [pattern]', 'List or describe tables, views and sequences.')
 def describe_table_details(cur, pattern, verbose):
     """
     Returns (title, rows, headers, status)
     """
 
-    # This is a simple \d command. No table name to follow.
+    # This is a simple \d[+] command. No table name to follow.
     if not pattern:
-        sql = """SELECT n.nspname as "Schema", c.relname as "Name",
-                    CASE c.relkind WHEN 'r' THEN 'table'
-                        WHEN 'v' THEN 'view'
-                        WHEN 'm' THEN 'materialized view'
-                        WHEN 'i' THEN 'index'
-                        WHEN 'S' THEN 'sequence'
-                        WHEN 's' THEN 'special'
-                        WHEN 'f' THEN 'foreign table'
-                    END as "Type",
-                    pg_catalog.pg_get_userbyid(c.relowner) as "Owner"
-                FROM pg_catalog.pg_class c
-                LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
-                WHERE c.relkind IN ('r','v','m','S','f','')
-                AND n.nspname <> 'pg_catalog'
-                AND n.nspname <> 'information_schema'
-                AND n.nspname !~ '^pg_toast'
-                AND pg_catalog.pg_table_is_visible(c.oid)
-                ORDER BY 1,2 """
-
-        log.debug(sql)
-        cur.execute(sql)
-        if cur.description:
-            headers = [x[0] for x in cur.description]
-            return [(None, cur, headers, cur.statusmessage)]
+        return list_objects(cur, pattern, verbose, ['r', 'v', 'm', 'S', 'f', ''])
 
     # This is a \d <tablename> command. A royal pain in the ass.
     schema, relname = sql_name_pattern(pattern)
@@ -656,8 +640,6 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
     cur.execute(sql)
     res = cur.fetchall()
 
-    title = (tableinfo.relkind, schema_name, relation_name)
-
     # Set the column names.
     headers = ['Column', 'Type']
 
@@ -996,7 +978,7 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
             for row in cur:
                 #/* untranslated contraint name and def */
                 status.append("    \"%s\" %s" % row)
-            status.append('\n')
+                status.append('\n')
 
         #/* print foreign-key constraints (there are none if no triggers) */
         if (tableinfo.hastriggers):
@@ -1310,3 +1292,42 @@ def sql_name_pattern(pattern):
         schema = '^(' + schema + ')$'
 
     return schema, relname
+
+
+class _FakeCursor(list):
+    "Minimalistic wrapper simulating a real cursor, as far as pgcli is concerned."
+
+    def rowcount(self):
+        return len(self)
+
+
+@special_command('\\sf', '\\sf[+] FUNCNAME', 'Show a function\'s definition.')
+def show_function_definition(cur, pattern, verbose):
+    if '(' in pattern:
+        sql = cur.mogrify("SELECT %s::pg_catalog.regprocedure::pg_catalog.oid", [pattern])
+    else:
+        sql = cur.mogrify("SELECT %s::pg_catalog.regproc::pg_catalog.oid", [pattern])
+    log.debug(sql)
+    cur.execute(sql)
+    (foid,) = cur.fetchone()
+
+    sql = cur.mogrify("SELECT pg_catalog.pg_get_functiondef(%s) as source", [foid])
+    log.debug(sql)
+    cur.execute(sql)
+    if cur.description:
+        headers = [x[0] for x in cur.description]
+        if verbose:
+            (source,) = cur.fetchone()
+            rows = _FakeCursor()
+            rown = None
+            for row in source.splitlines():
+                if rown is None:
+                    if row.startswith('AS '):
+                        rown = 1
+                else:
+                    rown += 1
+                rows.append('%-7s %s' % ('' if rown is None else rown, row))
+            cur = [('\n'.join(rows) + '\n',)]
+    else:
+        headers = None
+    return [(None, cur, headers, None)]
